#ifndef id873715F57B504CCF8227CE03EA28CAFA
#define id873715F57B504CCF8227CE03EA28CAFA

#include "primitivetypes.hpp"
#include <boost/iterator/iterator_facade.hpp>
#include <iterator>
#include <vector>

namespace dk {
	namespace implem {
		template <size_t D>
		inline size_t get_index_from_pos ( const Vector<CoordinateScalarType, D>& parPos, const Vector<CoordinateScalarType, D>& parSize ) a_pure;

#if defined(NDEBUG)
		template <>
		inline size_t get_index_from_pos<2> ( const Vector<CoordinateScalarType, 2>& parPos, const Vector<CoordinateScalarType, 2>& parSize ) a_pure;
#endif
	} //namespace implem

	template <typename T, size_t D>
	class TileIterator : public boost::iterator_facade<TileIterator<T, D>, T, boost::bidirectional_traversal_tag> {
		friend class boost::iterator_core_access;
	public:
		typedef Vector<CoordinateScalarType, D> coords;

		TileIterator ( void );
		TileIterator ( const TileIterator& parOther ) = default;
		TileIterator ( std::vector<T>* parData, const coords& parFrom, const coords& parTo );
		TileIterator ( std::vector<T>* parData, const coords& parFrom, const coords& parTo, const coords& parAreaFrom, const coords& parAreaTo, const coords& parTotal );
		~TileIterator ( void ) = default;

		const coords& position ( void ) const { return m_pos; }

	private:
		void increment ( void );
		void decrement ( void );
		void advance ( size_t parAdvance );
		ptrdiff_t distance_to ( const TileIterator& parOther );
		bool equal ( const TileIterator& parOther ) const;
		T& dereference ( void ) const { return (*m_data)[get_current_index()]; }
		size_t get_current_index ( void ) const { return implem::get_index_from_pos<D>(m_pos, m_total); }

		coords m_pos;
		coords m_from;
		coords m_to;
		coords m_areafrom;
		coords m_areato;
		coords m_total;
		std::vector<T>* m_data;
	};
} //namespace dk

#include "implem/tileiterator.inl"

#endif
